/*
 *  T602Filter.hxx: OpenOffice.org Uno import service for T602 documents
 *
 *  Copyright (C) 2004 Milan Broz     email: asi@seznam.cz
 *
 *  version 1.0 for OpenOffice.org 1.1.0
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _T602FILTER_HXX
#define _T602FILTER_HXX

#include <com/sun/star/ucb/XSimpleFileAccess.hpp>
#include <com/sun/star/xml/XImportFilter.hpp>

#include "attributelist.hxx"

#define IMPLEMENTATION_NAME "com.sun.star.comp.Writer.T602ImportFilter"

namespace XT602Filter {

typedef enum {L2,KAM,KOI} tcode;

typedef enum {
	standard,   // default
		fat,        // tucne
		cursive,    // kurziva
		bold,       // siroke
		tall,       // vysoke
		big,        // velke
		lindex,     // dolni index
		hindex,     // horni index
		err,        // jeste nenastaveno
		chngul      // zmena podtrzeni
} fonts; 

typedef enum {START,READCH,EOL,POCMD,EXPCMD,SETCMD,SETCH,WRITE,EEND,QUIT} tnode;

::rtl::OUString getImplementationName()
	throw ( ::com::sun::star::uno::RuntimeException );

::com::sun::star::uno::Sequence < ::rtl::OUString > getSupportedServiceNames()
	throw ( ::com::sun::star::uno::RuntimeException );


::com::sun::star::uno::Reference < ::com::sun::star::uno::XInterface > SAL_CALL 
	CreateInstance( const ::com::sun::star::uno::Reference< com::sun::star::lang::XMultiServiceFactory > &r)
	throw ( ::com::sun::star::uno::Exception );

 
//
// class XT602Filter
//

class XT602Filter : public cppu::WeakImplHelper1< com::sun::star::xml::XImportFilter>
{
private:
	::com::sun::star::uno::Reference<com::sun::star::io::XInputStream> xInputStream;
	::com::sun::star::uno::Reference<com::sun::star::lang::XMultiServiceFactory > xMSF;
	::com::sun::star::uno::Reference<com::sun::star::xml::sax::XDocumentHandler> xHandler;

	AttributeList *mpAttrList;
	
	tnode node;         // START
	
	struct {
		int   mt;       // radek pro hlavicku
		int   mb;       // radek pro paticku
		int   tb;       // tabulatory
		int   ct;       // kodovani 0-kamenici, 1-latin2, 2-koi8
		int   pn;       // od cisla stranky
		int   lh;       // radkovani 3-2x, 4-1.5x, 6-1x
		int   lm;       // levy okraj
		int   rm;       // pravy okraj
		int   pl;       // delka stranky
	} format602;
	
	struct { // inicializacni konstanty - nemeni se v prubehu konverze
		bool showcomm;      // true   zobrazit komentarove radky
		bool forcecode;     // false  uzivatel zmenil kodovani jinak nez je @CT
		tcode xcode;        // KAM    nastavene kodovani - forced
		bool ruscode;       // false  zapnuty tabulky pro azbuku
		bool reformatpars;  // false  preformatovat odstavce (mezery+zalomeni)
	} ini;
	
	struct { // font state - meni se podle fontu
		fonts nowfnt;     // soucasny font
		fonts oldfnt;     // predchozi font
		bool uline;       // pise se podrtrzene ?, lze pouzit i zmene fontu
		bool olduline;    // predchozi hodnota uline (zmena fontu)
	} fst;
	
	struct { // paragraph state
		int pars;            //       pocet koncu radku * radkovani na akt. strance
		bool comment;        //       jsem v komentari
		int wasspace;        // 0     byla mezera - pro reformat
		bool ccafterln;      // false
		bool willbeeop;      // false
		bool waspar;         // false
	} pst;
	
	void Reset602();
	unsigned char Readchar602();
	void Read602(); 
	void par602(bool endofpage);
	void inschr(unsigned char ch); 
	void inschrdef(unsigned char ch); 
	unsigned char Setformat602(char *cmd);
	int readnum(unsigned char *ch, bool show);
	tnode PointCmd602(unsigned char *ch);
	void setfnt(fonts fnt,bool mustwrite);
	void XT602Filter::wrtfnt();
	sal_Bool OptionsDlg();
	
	public:
		XT602Filter(const ::com::sun::star::uno::Reference<com::sun::star::lang::XMultiServiceFactory > &r ) :	xMSF( r ) {}
		virtual sal_Bool SAL_CALL importer(
			const com::sun::star::uno::Sequence<com::sun::star::beans::PropertyValue>& aSourceData,
			const com::sun::star::uno::Reference<com::sun::star::xml::sax::XDocumentHandler>& xxHandler,
			const com::sun::star::uno::Sequence<rtl::OUString>& msUserData)
			throw(::com::sun::star::uno::RuntimeException);
};

}

#endif
